<?php

namespace Drupal\taxonomy_fast_import\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\taxonomy\Entity\Vocabulary;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form constructor for the taxonomies import screen.
 *
 * @internal
 */
class TaxonomiesImportForm extends FormBase {

  /**
   * Entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a TaxonomiesImportForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'taxonomies_import_form';
  }

  /**
   * Builds the import form.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The render array for the form.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['help'] = [
      '#type' => 'markup',
      '#markup' => '<h3>Accepted formats for importation :</h3>
                    <p>1. One term per line <br/>
                    2. Children are marked with “-” (notion of depth)<br/>
                    Example : <br/>
                    Bird<br/>
                    - Crow<br/>
                    -- Black crow<br/>
                    --- Black crow with red eye<br/>
                    </p>
                    <h3>For deletion</h3>
                    <p>Load the vocabulary, the ID and the name of the terms will be separated by "|"<br/>
                    Delete the terms you want in the list, then check the "Overwrite" radio button and press import.
                    </p><hr>',
    ];

    $form['voca']['actions'] = [
      '#type' => 'actions',
    ];
    $form['voca']['vocabulary_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Name of the vocabulary if not exist in the list yet'),
    ];
    $form['voca']['vocabulary_system_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('System name of the vocabulary'),
    ];
    $form['voca']['actions']['submit2'] = [
      '#type' => 'submit',
      '#value' => $this->t('Create'),
      '#submit' => [[$this, 'submitFormTwo']],
    ];

    $options = [];
    $vocabularies = Vocabulary::loadMultiple();
    foreach ($vocabularies as $voc) {
      $options[$voc->id()] = $voc->label();
    }

    $form['vocabularies'] = [
      '#type' => 'select',
      '#title' => $this->t('Vocabularies'),
      '#options' => $options,
      '#ajax' => [
        'callback' => '::loadTaxoCallback',
        'disable-refocus' => FALSE,
        'event' => 'change',
        'wrapper' => 'edit-output',
        'progress' => [
          'type' => 'throbber',
          'message' => $this->t('Verifying entry...'),
        ],
      ],
    ];
    $form['term_list'] = [
      '#type' => 'textarea',
      '#title' => $this->t('List of terms to import'),
      '#rows' => 30,
      '#prefix' => '<div id="edit-output">',
      '#suffix' => '</div>',
    ];

    $form['overwrite'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Overwrite'),
      '#description' => $this->t('Careful: if checked, delete all terms which are NOT in the new list'),
    ];

    $form['actions2'] = [
      '#type' => 'actions',
    ];

    $form['actions2']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Import'),
    ];

    return $form;
  }

  /**
   * AJAX callback to load taxonomy terms.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The updated form element.
   */
  public function loadTaxoCallback(array &$form, FormStateInterface $form_state) {
    if ($selectedValue = $form_state->getValue('vocabularies')) {
      $terms = $this->entityTypeManager->getStorage('taxonomy_term')->loadTree($selectedValue);
      $form['term_list']['#value'] = $this->showTaxo(0, 0, $terms);
    }
    return $form['term_list'];
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Add validation logic if necessary.
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $data = $form_state->getValue('term_list');
    $data = explode("\r\n", (string) $data);
    $overwrite = $form_state->getValue('overwrite');
    $vocabulary = $form_state->getValue('vocabularies');
    $tabCurrentTerms = $termToDelete = $fields = $operations = [];
    $storage = $this->entityTypeManager->getStorage('taxonomy_term');

    if (!empty($data)) {
      foreach ($data as $row) {
        if (trim($row) != '') {
          $level = explode('-', $row);
          $count = count($level);
          $taxo = array_pop($level);
          $taxo = explode('|', $taxo);

          if (count($taxo) == 2) {
            $termToDelete[] = $taxo[0];
          }
          else {
            $levelsNew[$count] = $taxo[0];
            $fields = [
              'name' => $taxo[0],
              'vid' => $vocabulary,
              'parent' => !is_null($levelsNew[$count - 1]) ? [$levelsNew[$count - 1]] : [0],
            ];
          }
          if (!empty($fields)) {
            $op = '\Drupal\taxonomy_fast_import\Batch\TaxonomiesImportBatch::process';
            $operations[] = [$op, [$fields, $vocabulary]];
          }
        }
      }

      // Deleting items that are not in the new list.
      if ($overwrite) {
        $terms = $this->entityTypeManager->getStorage('taxonomy_term')->loadTree($vocabulary);
        foreach ($terms as $term) {
          $tabCurrentTerms[$term->tid] = $term->name;
        }

        // Compare old and new lists.
        $termToDelete = array_flip(array_unique($termToDelete));
        foreach ($tabCurrentTerms as $tid => $name) {
          if (!array_key_exists($tid, $termToDelete)) {
            if ($term = $storage->load($tid)) {
              $term->delete();
            }
          }
        }
      }

      if (!empty($fields)) {
        $batch = [
          'title' => t('Process import of taxonomies'),
          'operations' => $operations,
          'finished' => '\Drupal\taxonomy_fast_import\Batch\TaxonomiesImportBatch::finished',
          'init_message' => t('Initializing import.'),
          'error_message' => t('An error occurred during import.'),
        ];

        batch_set($batch);
      }
      else {
        $this->messenger()->addStatus(t('No new taxonomies imported, only deletion.'));
      }

    }
    else {
      $this->messenger()->addStatus(t('No taxonomies data to import.'));
    }

    $form_state->setRedirect('taxonomy_fast_import.quick_import');
  }

  /**
   * Submits the form to create a new vocabulary.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitFormTwo(array &$form, FormStateInterface $form_state) {
    $name = $form_state->getValue('vocabulary_name');
    $vid = $form_state->getValue('vocabulary_system_name');
    $vocabularies = $this->entityTypeManager->getStorage('taxonomy_vocabulary')->loadMultiple();

    if (!isset($vocabularies[$vid])) {
      $this->entityTypeManager->getStorage('taxonomy_vocabulary')->create([
        'vid' => $vid,
        'description' => '',
        'name' => $name,
      ])->save();
    }
    else {
      $this->messenger()->addMessage($name . ' vocabulary already exists.');
    }
  }

  /**
   * Recursive method to show the terms tree imbrication.
   *
   * @param int $parent
   *   The parent term ID.
   * @param int $level
   *   The current depth level.
   * @param array $array
   *   The array of terms.
   *
   * @return string
   *   The rendered HTML string of terms.
   */
  public function showTaxo($parent, $level, $array) {
    $html = "";
    foreach ($array as $term) {
      if ($parent == $term->parents[0]) {
        for ($i = 0; $i < $level; $i++) {
          $html .= "-";
        }
        $html .= $term->tid . "|" . $term->name . "\r\n";
        $html .= $this->showTaxo($term->tid, ($level + 1), $array);
      }
    }

    return $html;
  }

}
